from .base import BaseNotificationChannel


class TelegramNotificationChannel(BaseNotificationChannel):
    """
    structure: []
    for example:
    ["user_1", "user_2", "user_3", ]
    """
    _receivers = []

    def __init__(self, **kwargs):
        super(TelegramNotificationChannel, self).__init__(**kwargs)

    def _send_msg(self, message, users):
        sender = self.reader.get_telegram_sender(self.identifier)
        sender.bulk_send(users, message)

    def get_message(self):
        return self.raw_message

    def set_receivers(self, users: list):
        self._receivers = users

    def get_receivers(self):
        return self._receivers

    def notify(self):
        """Sends the notification."""
        self._send_msg(message=self.get_message(), users=self.get_receivers())



import abc
import six


@six.add_metaclass(abc.ABCMeta)
class TelegramApi:

    def __init__(self, **kwargs):
        pass

    @abc.abstractmethod
    def send(self, username: str, message: str):
        pass

    @abc.abstractmethod
    def bulk_send(self, usernames: [str], message: str):
        pass



from azmessaging import default_settings as settings


class TelegramConfig:
    default_service_provider: str
    priorities_service_provider: [str]
    service_providers: dict

    def __init__(self, default, priorities, service_providers):
        self.default_service_provider = default
        self.priorities_service_provider = priorities
        self.service_providers = service_providers

    @classmethod
    def get_service_provider_class_path(cls, service_provider_name) -> str:
        return settings.TELEGRAM_CONFIG['SERVICE_PROVIDER_CLASS'].get(service_provider_name, None)



from .telegramabstract import TelegramApi


class TELEGRAMAPIDefaultAPI(TelegramApi):

    def __init__(self, api_key, api_server, **kwargs):
        super(TELEGRAMAPIDefaultAPI, self).__init__(**kwargs)
        self.api_key = api_key
        self.api_server = api_server

    def bulk_send(self, usernames: [str], message: str):
        # TODO: handle bulk options
        for username in usernames:
            self.send(username, message)

    def send(self, username: str, message: str):
        # TODO: handle send by your self
        raise NotImplementedError("You must Implement by your self to send to telegram")
